//CW3DSkyBox.h
//Brent Gunning
//Copyright 2002 All Rights Reserved

//holds a class for rendering skyboxes. skyboxes are cubes that surround
//you on all sides and are textured to look like an environment. As you move
//around your world, the skybox also moves.

//Note: The setup of the vertices and texture are as follows:
//top, bottom, left, right, front, back

//define the file
#ifndef CW3DSKYBOX_H
#define CW3DSKYBOX_H

//include files
#include "d3d8.h"
#include "d3dx8.h"
#include "VertexFormats.h"

//skybox vertex define
#define SKYBOXVERTEX VERTEX_XYZ_DIF_TEX
#define D3DFVF_SKYBOXVERTEX D3DFVF_VERTEX_XYZ_DIF_TEX

//the actual skybox class. it creates and renders a skybox
class CW3DSkyBox
{
public:
	//constructor/deconstructor
	//sets defaults and releases memory
	CW3DSkyBox();
	~CW3DSkyBox();

	//initializes the skybox. it takes a vector describing the size of the
	//box in width, height and depth. It also takes the filenames of the 6
	//textures. It basically just loads the textures and sets up the vbuffer
	HRESULT Initialize(D3DXVECTOR3 vecSize,char* strTop,
						char* strBottom,char* strLeft,
						char* strRight,char* strFront,char* strBack,
						IDirect3DDevice8* pDevice);

	//this destroys the class's memory. It releases the vbuffer and 6 textures
	HRESULT Destroy();

	//renders the skybox. it takes the position of the camera so it can align it
	//to the skybox's position
	HRESULT Render(D3DXVECTOR3 vecCameraPosition,IDirect3DDevice8* pDevice);
protected:
	//the vertex buffer to hold the skybox vertices
	IDirect3DVertexBuffer8* m_VertexBuffer;
	//the textures for the skybox
	IDirect3DTexture8* m_Textures[6];
	//error variable
	HRESULT hr;
};

#endif
