//CW3DCamera.cpp
//Brent Gunning
//Copyright 2002 All Rights Reserved

#include "../headers/CW3DCamera.h"

//constructor/deconstructor
//sets matrix identities
CW3DCamera::CW3DCamera()
{
	//set defaults
	D3DXMatrixIdentity(&m_matView);
	D3DXMatrixIdentity(&m_matProj);
	m_vecPos=D3DXVECTOR3(0,0,0);
	m_vecRot=D3DXVECTOR3(0,0,0);
	hr=0;
}
CW3DCamera::~CW3DCamera()
{
	//it needs no releasing of memory
}
//Initializes the view and projection matrices
void CW3DCamera::Initialize(int Width,int Height,D3DXVECTOR3 vecPos,D3DXVECTOR3 vecRot)
{
	//setup the projection matrix
	float dim=(float)Width/(float)Height;
	D3DXMatrixPerspectiveLH(&m_matProj,dim,1.0f,1.0f,10000.0f);
	
	//setup the view matrix
	m_vecPos=vecPos;
	m_vecRot=vecRot;
	Update();
}
//updates the matrices
void CW3DCamera::Update()
{
	//translate the camera
	D3DXMATRIX matTrans;
	D3DXMatrixTranslation(&matTrans,-m_vecPos.x,-m_vecPos.y,-m_vecPos.z);
	//rotate the camera
	D3DXMATRIX matRot,matRotTemp;
	D3DXMatrixRotationX(&matRot,-m_vecRot.x);
	D3DXMatrixRotationY(&matRotTemp,-m_vecRot.y);
	D3DXMatrixMultiply(&matRot,&matRot,&matRotTemp);
	D3DXMatrixRotationZ(&matRotTemp,-m_vecRot.z);
	D3DXMatrixMultiply(&matRot,&matRot,&matRotTemp);

	//multiply the matrices together
	D3DXMatrixMultiply(&m_matView,&matTrans,&matRot);
}
//assigns the camera to the device
HRESULT CW3DCamera::Assign(IDirect3DDevice8* pDevice)
{
	//assign the projection and view matrices
	if(FAILED(hr=pDevice->SetTransform(D3DTS_VIEW,&m_matView)))
		return(hr);
	if(FAILED(hr=pDevice->SetTransform(D3DTS_PROJECTION,&m_matProj)))
		return(hr);

	//return success
	return(S_OK);
}