//CW3DFrustrum.cpp
//Brent Gunning
//Copyright 2002 All Rights Reserved

#include "../headers/CW3DFrustum.h"

//constructor/deconstructor
//like most others, releases memory and sets defaults
CW3DFrustum::CW3DFrustum()
{
	//set defaults
	hr=0;
	
}
CW3DFrustum::~CW3DFrustum()
{
	//release the memory
	Destroy();
}
//creates the viewing frustum based on the matrices it extracts
HRESULT CW3DFrustum::Initialize(D3DXMATRIX matCombined)
{
	//left plane
	m_Planes[PLANESIDE_LEFT].a=matCombined._14+matCombined._11;
	m_Planes[PLANESIDE_LEFT].b=matCombined._24+matCombined._21;
	m_Planes[PLANESIDE_LEFT].c=matCombined._34+matCombined._31;
	m_Planes[PLANESIDE_LEFT].d=matCombined._44+matCombined._41;
	//right plane
	m_Planes[PLANESIDE_RIGHT].a=matCombined._14-matCombined._11;
	m_Planes[PLANESIDE_RIGHT].b=matCombined._24-matCombined._21;
	m_Planes[PLANESIDE_RIGHT].c=matCombined._34-matCombined._31;
	m_Planes[PLANESIDE_RIGHT].d=matCombined._44-matCombined._41;
	//top plane
	m_Planes[PLANESIDE_TOP].a=matCombined._14-matCombined._12;
	m_Planes[PLANESIDE_TOP].b=matCombined._24-matCombined._22;
	m_Planes[PLANESIDE_TOP].c=matCombined._34-matCombined._32;
	m_Planes[PLANESIDE_TOP].d=matCombined._44-matCombined._42;
	//bottom plane
	m_Planes[PLANESIDE_BOTTOM].a=matCombined._14+matCombined._12;
	m_Planes[PLANESIDE_BOTTOM].b=matCombined._24+matCombined._22;
	m_Planes[PLANESIDE_BOTTOM].c=matCombined._34+matCombined._32;
	m_Planes[PLANESIDE_BOTTOM].d=matCombined._44+matCombined._42;
	//near plane
	m_Planes[PLANESIDE_NEAR].a=matCombined._14+matCombined._13;
	m_Planes[PLANESIDE_NEAR].b=matCombined._24+matCombined._23;
	m_Planes[PLANESIDE_NEAR].c=matCombined._34+matCombined._33;
	m_Planes[PLANESIDE_NEAR].d=matCombined._44+matCombined._43;
	//far plane
	m_Planes[PLANESIDE_FAR].a=matCombined._14-matCombined._13;
	m_Planes[PLANESIDE_FAR].b=matCombined._24-matCombined._23;
	m_Planes[PLANESIDE_FAR].c=matCombined._34-matCombined._33;
	m_Planes[PLANESIDE_FAR].d=matCombined._44-matCombined._43;

	//normalize the planes
	for(int i=0;i<6;i++)
		m_Planes[i].Normalize();

	//return success
	return(S_OK);
}
//checks if a point is in the frustum
bool CW3DFrustum::IsPointInFrustum(float x,float y, float z)
{
	//loop and check points
	for(int i=0;i<6;i++)
		if(m_Planes[i].PointDistance(x,y,z)<0)
		{
			return(false);
		}
	//return true
	return(true);
}
//checks if a cube is in the frustum
bool CW3DFrustum::IsBoxInFrustum(float x,float y,float z,float width,float height,float depth)
{
	for(int i=0;i<6;i++)
	{
		if(m_Planes[i].PointDistance(x-width,y-height,z-depth)>0)
			continue;
		if(m_Planes[i].PointDistance(x+width,y-height,z-depth)>0)
			continue;
		if(m_Planes[i].PointDistance(x-width,y+height,z-depth)>0)
			continue;
		if(m_Planes[i].PointDistance(x+width,y+height,z-depth)>0)
			continue;
		if(m_Planes[i].PointDistance(x-width,y-height,z+depth)>0)
			continue;
		if(m_Planes[i].PointDistance(x+width,y-height,z+depth)>0)
			continue;
		if(m_Planes[i].PointDistance(x-width,y+height,z+depth)>0)
			continue;
		if(m_Planes[i].PointDistance(x+width,y+height,z+depth)>0)
			continue;
		return(false);
	}
	//return true
	return(true);
}
//checks if a sphere is in the frustum
bool CW3DFrustum::IsSphereInFrustum(float x,float y,float z,float radius)
{
	for(int i=0;i<6;i++)
	{
		if(m_Planes[i].PointDistance(x,y,z)<-radius)
			return(false);
	}
	//return true
	return(true);
}
//releases the frustum
HRESULT CW3DFrustum::Destroy()
{
	//loop and release planes
	for(int i=0;i<6;i++)
		if(FAILED(hr=m_Planes[i].Release()))
			return(hr);

	//return success
	return(S_OK);
}