//CW3DParticleSystem.cpp
//Brent Gunning
//Copyright 2002 All Rights Reserved

#include "../headers/CW3DParticleSystem.h"

//constructor/deconstructor
//they just release the memory and set 0's
CW3DParticleSystem::CW3DParticleSystem()
{
	//set 0's and clear structures
	ZeroMemory(&m_psp,sizeof(sParticleSysProps));
	m_VB=0;
	m_pParticles=0;
	hr=0;
}
CW3DParticleSystem::~CW3DParticleSystem()
{
	//call destroy to release the system's memory
	Destroy();
}
//will create the particles baesd on the particle
//system properties structure given.
HRESULT CW3DParticleSystem::Initialize(sParticleSysProps psp,IDirect3DDevice8* pDevice)
{
	//copy the structures
	memcpy(&m_psp,&psp,sizeof(sParticleSysProps));

	//create the new particles
	m_pParticles=new sParticle[psp.iNumParticles];

	//create the vertex buffer to hold the particles
	//we make it dynamic because we'll be doing a lot
	//of updating on it
	if(FAILED(hr=pDevice->CreateVertexBuffer(sizeof(PARTICLEVERTEX)*psp.iNumParticles*3,//size of vbuffer
								D3DUSAGE_DYNAMIC|D3DUSAGE_WRITEONLY,//usage
								D3DFVF_PARTICLEVERTEX,
								D3DPOOL_DEFAULT,//memory pool
								&m_VB)))
			return(hr);

	//loop through the particles and set the default properties
	for(int i=0;i<psp.iNumParticles;i++)
	{
		//determine the velocity randomness values
		D3DXVECTOR3 vr;
		vr.x=RandomFloat(m_psp.vecInitVel.x-m_psp.vecVelRand.x,m_psp.vecInitVel.x+m_psp.vecVelRand.x);
		vr.y=RandomFloat(m_psp.vecInitVel.y-m_psp.vecVelRand.y,m_psp.vecInitVel.y+m_psp.vecVelRand.y);
		vr.z=RandomFloat(m_psp.vecInitVel.z-m_psp.vecVelRand.z,m_psp.vecInitVel.z+m_psp.vecVelRand.z);
		//determine the lifetime randomness
		int lr=(int)RandomFloat((float)(m_psp.iLifetime-m_psp.iLifetimeRand),(float)(m_psp.iLifetime+m_psp.iLifetimeRand));
		//find the weight randomness
		float wr=RandomFloat(m_psp.fWeight-m_psp.fWeightRand,m_psp.fWeight+m_psp.fWeightRand);
		//create the position randomness
		D3DXVECTOR3 pr;
		pr.x=RandomFloat(m_psp.vecInitPos.x-m_psp.vecPosRand.x,m_psp.vecInitPos.x+m_psp.vecPosRand.x);
		pr.y=RandomFloat(m_psp.vecInitPos.y-m_psp.vecPosRand.y,m_psp.vecInitPos.y+m_psp.vecPosRand.y);
		pr.z=RandomFloat(m_psp.vecInitPos.z-m_psp.vecPosRand.z,m_psp.vecInitPos.z+m_psp.vecPosRand.z);
			
		//set the particle properties
		m_pParticles[i].pos=pr;
		m_pParticles[i].vel=vr;
		m_pParticles[i].life=lr;
		m_pParticles[i].weight=wr;
		m_pParticles[i].color=m_psp.cStartColor;
		m_pParticles[i].bAlive=true;
	}

	//return success
	return(S_OK);
}
//will update the particles. updates various things
//such as the color, positon, etc. to create the billboards
//from them
HRESULT CW3DParticleSystem::Process()
{
	//first check if it was initialized
	if(m_VB==0 || m_pParticles==0)
		return(E_FAIL);

	//loop through all the particles to update
	for(int i=0;i<m_psp.iNumParticles;i++)
	{
		//check if the particle is alive. if it's not, recreate it if allowed
		if(m_pParticles[i].bAlive==false && m_psp.bRemakeParticles==true)
		{
			//determine the velocity randomness values
			D3DXVECTOR3 vr;
			vr.x=RandomFloat(m_psp.vecInitVel.x-m_psp.vecVelRand.x,m_psp.vecInitVel.x+m_psp.vecVelRand.x);
			vr.y=RandomFloat(m_psp.vecInitVel.y-m_psp.vecVelRand.y,m_psp.vecInitVel.y+m_psp.vecVelRand.y);
			vr.z=RandomFloat(m_psp.vecInitVel.z-m_psp.vecVelRand.z,m_psp.vecInitVel.z+m_psp.vecVelRand.z);
			//determine the lifetime randomness
			int lr=(int)RandomFloat((float)(m_psp.iLifetime-m_psp.iLifetimeRand),(float)(m_psp.iLifetime+m_psp.iLifetimeRand));
			//find the weight randomness
			float wr=RandomFloat(m_psp.fWeight-m_psp.fWeightRand,m_psp.fWeight+m_psp.fWeightRand);
			//create the position randomness
			D3DXVECTOR3 pr;
			pr.x=RandomFloat(m_psp.vecInitPos.x-m_psp.vecPosRand.x,m_psp.vecInitPos.x+m_psp.vecPosRand.x);
			pr.y=RandomFloat(m_psp.vecInitPos.y-m_psp.vecPosRand.y,m_psp.vecInitPos.y+m_psp.vecPosRand.y);
			pr.z=RandomFloat(m_psp.vecInitPos.z-m_psp.vecPosRand.z,m_psp.vecInitPos.z+m_psp.vecPosRand.z);
			
			//set the particle properties
			m_pParticles[i].pos=pr;
			m_pParticles[i].vel=vr;
			m_pParticles[i].life=lr;
			m_pParticles[i].weight=wr;
			m_pParticles[i].color=m_psp.cStartColor;
			m_pParticles[i].bAlive=true;
		}
		else//it's alive so lets update it
		{
			//add the velocity to the position to move the particle
			m_pParticles[i].pos+=m_pParticles[i].vel;

			//subtract the weight from the velocity
			m_pParticles[i].vel.y-=m_pParticles[i].weight;

			//add the wind to the position
			m_pParticles[i].pos+=m_psp.vecWind;

			//subtract a life from the particle life so that it will eventuall die
			m_pParticles[i].life--;

			//check if the particle exists
			if(m_pParticles[i].life<0)
				m_pParticles[i].bAlive=false;

			//linearly interpolate between the colors
			float interpolation=(float)(m_psp.iLifetime-m_pParticles[i].life)/(float)m_psp.iLifetime;
			//check that it's not less than 0
			if(interpolation<0)
				interpolation=0;

			//using the formula o=a+i*(b-a), linear interpolate between the colors
			D3DXCOLOR out=m_psp.cStartColor+interpolation*(m_psp.cEndColor-m_psp.cStartColor);

			//set the final color. we multiply the values by 255 because they are from 0-1
			m_pParticles[i].color=D3DCOLOR_ARGB((int)(out.a*255),(int)(out.r*255),(int)(out.g*255),(int)(out.b*255));
		}
	}
	//now we need to lock the vertex buffer and create the vertices
	PARTICLEVERTEX* pData;
	if(FAILED(hr=m_VB->Lock(0,0,(BYTE**)&pData,D3DLOCK_DISCARD)))
		return(hr);

	//loop through the particles/vertices
	for(i=0;i<m_psp.iNumParticles;i++)
	{
		//check if the particle is alive. if it is, create it. if not, clear memory
		if(m_pParticles[i].bAlive)
		{
			//vert1
			pData[i*3].pos=D3DXVECTOR3(	m_pParticles[i].pos.x-m_psp.fXSize,
												m_pParticles[i].pos.y-m_psp.fYSize,
												m_pParticles[i].pos.z);
			pData[i*3].nrm=D3DXVECTOR3(0,0,-1);
			pData[i*3].dif=m_pParticles[i].color;
			pData[i*3].tu=pData[i*3].tv=0;

			//vert2
			pData[i*3+1].pos=D3DXVECTOR3(	m_pParticles[i].pos.x+m_psp.fXSize,
												m_pParticles[i].pos.y-m_psp.fYSize,
												m_pParticles[i].pos.z);
			pData[i*3+1].nrm=D3DXVECTOR3(0,0,-1);
			pData[i*3+1].dif=m_pParticles[i].color;
			pData[i*3+1].tu=1;
			pData[i*3+1].tv=0;

			//vert3
			pData[i*3+2].pos=D3DXVECTOR3(	m_pParticles[i].pos.x+m_psp.fXSize,
												m_pParticles[i].pos.y+m_psp.fYSize,
												m_pParticles[i].pos.z);
			pData[i*3+2].nrm=D3DXVECTOR3(0,0,-1);
			pData[i*3+2].dif=m_pParticles[i].color;
			pData[i*3+2].tu=1;
			pData[i*3+2].tv=1;
		}
		else
		{
			ZeroMemory(&pData[i*3+0],sizeof(PARTICLEVERTEX));
			ZeroMemory(&pData[i*3+1],sizeof(PARTICLEVERTEX));
			ZeroMemory(&pData[i*3+2],sizeof(PARTICLEVERTEX));
		}
	}

	//unlock the buffer because we're finished modifying it
	if(FAILED(hr=m_VB->Unlock()))
		return(hr);

	//return success
	return(S_OK);
}
//releases the memory for the class
HRESULT CW3DParticleSystem::Destroy()
{
	if(m_VB)
	{
		m_VB->Release();
		m_VB=0;
	}
	if(m_pParticles)
	{
		delete[] m_pParticles;
		m_pParticles=0;
	}
	//return success
	return(S_OK);
}
//it will render the particles using a vertex buffer. it takes the view
//and world matrix because it needs 
HRESULT CW3DParticleSystem::Render(IDirect3DDevice8* pDevice,D3DXMATRIX matWorld)
{
	//first check if it was initialized
	if(m_VB==0 || m_pParticles==0)
		return(E_FAIL);

	//set the billboard matrix
	if(FAILED(hr=pDevice->SetTransform(D3DTS_WORLD,&matWorld)))
		return(hr);

	//render the particle buffer
	if(FAILED(hr=pDevice->SetStreamSource(0,m_VB,sizeof(PARTICLEVERTEX))))
		return(hr);
	if(FAILED(hr=pDevice->SetVertexShader(D3DFVF_PARTICLEVERTEX)))
		return(hr);
	if(FAILED(hr=pDevice->DrawPrimitive(D3DPT_TRIANGLELIST,0,m_psp.iNumParticles)))
		return(hr);

	//return success
	return(S_OK);
}
//function to generate a random number
//used with the randomness variables
float CW3DParticleSystem::RandomFloat(float low,float hi)
{
	//first find a floating point number between 0-1 which is random
	float fRandom=(float)rand()/(float)RAND_MAX;

	//based on the hi and low values, find the random number and return it
	return((fRandom*(float)fabs(hi-low))+low);
}