//MilkshapeLoader.cpp
//Brent Gunning
//Copyright 2002 All Rights Reserved

//include header
#include "../headers/MilkshapeLoader.h"

//this is the actual function to load the bones into the milkshape data structure
sMilkshapeData* LoadMilkshapeModel(char* strFilename)
{
	//create the new data
	sMilkshapeData* pData=new sMilkshapeData();

	//variables
	FILE* file;		//file
	int i=0;		//index
	sMS3DHeader header;
	sMS3DNumbers numbers;

	//open binary file
	file=fopen(strFilename,"rb");
	if(file==0)
		return(0);

	//read in the header and verify
	fread(&header.ID,sizeof(char)*10,1,file);
	fread(&header.Version,sizeof(int),1,file);

	if(strncmp(header.ID,"MS3D000000",10)!=0)
	{
		delete pData;
		return(0);
	}
	if(header.Version<3)
	{
		delete pData;
		return(0);
	}

	//read in number of vertices and create
	fread(&numbers.NumVertices,sizeof(WORD),1,file);
	pData->NumVertices=numbers.NumVertices;
	pData->pVertices=new sVertex[numbers.NumVertices];

	//loop and load vertices
	for(i=0;i<numbers.NumVertices;i++)
	{
		sMS3DVertex Vertex;
		fread(&Vertex.Flags,sizeof(BYTE),1,file);
		fread(&Vertex.Position[0],sizeof(float),1,file);		
		fread(&Vertex.Position[1],sizeof(float),1,file);		
		fread(&Vertex.Position[2],sizeof(float),1,file);		
		fread(&Vertex.BoneID,sizeof(char),1,file);	
		fread(&Vertex.ReferenceCount,sizeof(BYTE),1,file);	

		//set the vertex data
		pData->pVertices[i].vecPos.x=Vertex.Position[0];
		pData->pVertices[i].vecPos.y=Vertex.Position[1];
		pData->pVertices[i].vecPos.z=-Vertex.Position[2];
		if(Vertex.BoneID<0)
			Vertex.BoneID=-1;
		pData->pVertices[i].BoneIndex=(int)Vertex.BoneID;
	}

	//read in the number of triangles and create
	fread(&numbers.NumTriangles,sizeof(WORD),1,file);
	pData->NumFaces=numbers.NumTriangles;
	pData->pFaces=new sFace[numbers.NumTriangles];

	//loop and read in the triangles
	for(i=0;i<numbers.NumTriangles;i++)
	{
		sMS3DTriangle Triangle;
		fread(&Triangle.Flags,sizeof(WORD),1,file);
		fread(&Triangle.VertexIndices,sizeof(WORD)*3,1,file);
		fread(&Triangle.VertexNormals,sizeof(float)*9,1,file);	
		fread(&Triangle.s,sizeof(float)*3,1,file);	
		fread(&Triangle.t,sizeof(float)*3,1,file);	
		fread(&Triangle.SmoothingGroup,sizeof(BYTE),1,file);
		fread(&Triangle.GroupIndex,sizeof(BYTE),1,file);

		//copy face data into data structure
		for(int j=0;j<3;j++)
		{
			//vert normals
			pData->pFaces[i].VertexNormals[j].x=Triangle.VertexNormals[j][0];
			pData->pFaces[i].VertexNormals[j].y=Triangle.VertexNormals[j][1];
			pData->pFaces[i].VertexNormals[j].z=-Triangle.VertexNormals[j][2];

			//vertex indices
			pData->pFaces[i].VertexIndices[j]=(int)Triangle.VertexIndices[j];

			//tex coords
			pData->pFaces[i].tu[j]=Triangle.s[j];
			pData->pFaces[i].tv[j]=Triangle.t[j];
		}
	}

	//read in the number of groups and create
	fread(&numbers.NumGroups,sizeof(WORD),1,file);
	pData->NumGroups=numbers.NumGroups;
	pData->pGroups=new sGroup[numbers.NumGroups];

	//loop and read in the mesh groups
	for(i=0;i<numbers.NumGroups;i++)
	{
		sMS3DGroup group;
		fread(&group.Flags,sizeof(BYTE),1,file);
		fseek(file,32,SEEK_CUR);
		fread(&group.NumTriangles,sizeof(WORD),1,file);
		group.TriangleIndices=new WORD[group.NumTriangles];
		for(int index=0;index<group.NumTriangles;index++)
			fread(&group.TriangleIndices[index],sizeof(WORD),1,file);
		fread(&group.MaterialIndex,sizeof(char),1,file);

		//create and copy data from group to data group
		pData->pGroups[i].NumFaces=group.NumTriangles;
		pData->pGroups[i].MaterialIndex=(int)group.MaterialIndex;
		pData->pGroups[i].pFaceIndices=new int[group.NumTriangles];
		for(int j=0;j<group.NumTriangles;j++)
		{
			pData->pGroups[i].pFaceIndices[j]=(int)group.TriangleIndices[j];
		}
	}

	//read the number of materials and create
	fread(&numbers.NumMaterials,sizeof(WORD),1,file);
	pData->NumMaterials=numbers.NumMaterials;
	pData->pMaterials=new sMaterial[numbers.NumMaterials];

	//read in the materials
	for(i=0;i<numbers.NumMaterials;i++)
	{
		sMS3DMaterial material;
		fread(&material.Name,sizeof(char)*32,1,file);
		fread(&material.Ambient,sizeof(float)*4,1,file);
		fread(&material.Diffuse,sizeof(float)*4,1,file);
		fread(&material.Specular,sizeof(float)*4,1,file);
		fread(&material.Emissive,sizeof(float)*4,1,file);
		fread(&material.Shininess,sizeof(float),1,file);
		fread(&material.Transparency,sizeof(float),1,file);
		fread(&material.Mode,sizeof(BYTE),1,file);
		fread(&material.TextureFilename,sizeof(char)*128,1,file);
		fread(&material.AlphamapFilename,sizeof(char)*128,1,file);

		//copy material
		pData->pMaterials[i].Material.Ambient.r=material.Ambient[0];//Ambient
		pData->pMaterials[i].Material.Ambient.g=material.Ambient[1];
		pData->pMaterials[i].Material.Ambient.b=material.Ambient[2];
		pData->pMaterials[i].Material.Ambient.a=material.Ambient[3];
		pData->pMaterials[i].Material.Diffuse.r=material.Diffuse[0];//Diffuse
		pData->pMaterials[i].Material.Diffuse.g=material.Diffuse[1];
		pData->pMaterials[i].Material.Diffuse.b=material.Diffuse[2];
		pData->pMaterials[i].Material.Diffuse.a=material.Diffuse[3];
		pData->pMaterials[i].Material.Specular.r=material.Specular[0];//Specular
		pData->pMaterials[i].Material.Specular.g=material.Specular[1];
		pData->pMaterials[i].Material.Specular.b=material.Specular[2];
		pData->pMaterials[i].Material.Specular.a=material.Specular[3];
		pData->pMaterials[i].Material.Emissive.r=material.Emissive[0];//Emissive
		pData->pMaterials[i].Material.Emissive.g=material.Emissive[1];
		pData->pMaterials[i].Material.Emissive.b=material.Emissive[2];
		pData->pMaterials[i].Material.Emissive.a=material.Emissive[3];
		pData->pMaterials[i].Material.Power=material.Shininess;//power

		//create new texture name and copy
		pData->pMaterials[i].TextureFilename=new char[128];
		strcpy(pData->pMaterials[i].TextureFilename,material.TextureFilename);
	}

	//close file
	fclose(file);

	//return the data
	return(pData);
}