//CW3DGame.cpp
//Brent Gunning
//Copyright 2002 All Rights Reserved

//include the header file for this class
#include "../headers/CW3DGame.h"

//this will create and initialize all
//game objects. it is called as soon
//as the window is created in main.cpp
bool CW3DGame::Initialize(HWND hWnd)
{
	//first create the device for rendering with 800x600 resolution
	if(FAILED(m_Device.Initialize(hWnd,VIEW_WIDTH,VIEW_HEIGHT,WINDOWED)))
		return(false);

	//get the device pointer
	IDirect3DDevice8* pDevice=m_Device.GetDevice();

	//set default render states.
	if(FAILED(pDevice->SetRenderState(D3DRS_LIGHTING,false)))
		return(false);
	if(FAILED(pDevice->SetRenderState(D3DRS_CULLMODE,D3DCULL_NONE)))
		return(false);
	if(FAILED(pDevice->SetRenderState(D3DRS_ZENABLE,true)))
		return(false);
	if(FAILED(pDevice->SetRenderState(D3DRS_EDGEANTIALIAS,true)))
		return(false);

	//initialize the camera. it creates the view and projection matrices
	m_Camera.Initialize(VIEW_WIDTH,VIEW_HEIGHT,D3DXVECTOR3(-0,8,-0),D3DXVECTOR3(0.0f,3.14f,0));

	//initialize the entire world
	if(FAILED(m_World.Initialize(pDevice)))
		return(false);

	//initialize direct input and all control objects
	if(FAILED(m_Control.Initialize(hWnd)))
		return(false);
	//return success as default
	return(true);
}
//processes the data. you could also call
//this the main game loop function. it does
//everything. there is no reason to return 
//a value.
void CW3DGame::Process()
{
	//get the device pointer
	IDirect3DDevice8* pDevice=m_Device.GetDevice();

	//process the control and get certain values
	if(FAILED(m_Control.Process()))
		return;
	float angle;
	BOOL up,left,right;
	D3DXVECTOR3 vecPos;
	m_Control.GetValues(vecPos,angle,up,left,right);

	//update the camera based on the control values
	m_Camera.SetPosition(vecPos+D3DXVECTOR3(0,CAMERA_DISTANCE,-CAMERA_DISTANCE));
	m_Camera.SetRotation(D3DXVECTOR3(0,angle*3.14f/180.0f,0));
	m_Camera.Update();
	m_Camera.Assign(pDevice);

	//clear the screen first
	pDevice->Clear(0,0,D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER,CLEAR_COLOR,1.0f,0);

	//prepare to render by calling begin scene
	m_Device.BeginScene();

	//render the world
	m_World.Render(pDevice,&m_Camera);
	
	//tell d3d to stop rendering with end scene
	m_Device.EndScene();

	//present the scene to the screen
	m_Device.Present();
}
//cleans up and releases memory. it's called just
//before the program quits
void CW3DGame::Destroy()
{
	//destroy the world
	m_World.Destroy();
	//destroy the device
	m_Device.Destroy();
	//release the control object
	m_Control.Destroy();
}