//CW3DWater.cpp
//Brent Gunning
//Copyright 2002 All Rights Reserved

//include header file
#include "../headers/CW3DWater.h"

//constructor/deconstructor
//sets defaults and releases memory
CW3DWater::CW3DWater()
{
	//set default memory
	m_VertexBuffer=0;
	m_NumVerts=0;
	hr=0;
	m_WaterTexture=m_FloorTexture=0;
}
CW3DWater::~CW3DWater()
{
	//release the class's memory sinse we're deconstructing
	Destroy();
}
//initializes the water. it sets up the vertex buffers for rendering
///returns an hresult for error testing
HRESULT CW3DWater::Initialize(IDirect3DDevice8* pDevice)
{
	//first, destroy the memory for the class if it was already created
	Destroy();

	//determine the number of vertices
	m_NumVerts=8;

	//now we create a dynamic vertex buffer so that we can update it with the waves
	//we make it write only because we'll only be copying the vertices into it,
	//never reading from it
	if(FAILED(hr=pDevice->CreateVertexBuffer(m_NumVerts*sizeof(WATERVERTEX),//size
											D3DUSAGE_WRITEONLY,//usage
											D3DFVF_WATERVERTEX,
											D3DPOOL_DEFAULT,
											&m_VertexBuffer)))
			return(hr);

	//now setup the initial vertex buffer values.
	//now we lock the vbuffer
	WATERVERTEX* pVertexData;
	if(FAILED(hr=m_VertexBuffer->Lock(0,0,(BYTE**)&pVertexData,0)))
		return(hr);
	
	//set the first four vertices to plane vertices.
	pVertexData[0].pos=D3DXVECTOR3(-FLOOR_LENGTH,FLOOR_HEIGHT,-FLOOR_WIDTH);
	pVertexData[0].nrm=D3DXVECTOR3(0,1,0);
	pVertexData[0].dif=D3DCOLOR_ARGB(255,255,255,255);
	pVertexData[0].tu=0;
	pVertexData[0].tv=0;
	pVertexData[1].pos=D3DXVECTOR3(FLOOR_LENGTH,FLOOR_HEIGHT,-FLOOR_WIDTH);
	pVertexData[1].nrm=D3DXVECTOR3(0,1,0);
	pVertexData[1].dif=D3DCOLOR_ARGB(255,255,255,255);
	pVertexData[1].tu=NUM_FLOOR_WIDTH_WRAPS;
	pVertexData[1].tv=0;
	pVertexData[2].pos=D3DXVECTOR3(-FLOOR_LENGTH,FLOOR_HEIGHT,FLOOR_WIDTH);
	pVertexData[2].nrm=D3DXVECTOR3(0,1,0);
	pVertexData[2].dif=D3DCOLOR_ARGB(255,255,255,255);
	pVertexData[2].tu=0;
	pVertexData[2].tv=NUM_FLOOR_WIDTH_WRAPS;
	pVertexData[3].pos=D3DXVECTOR3(FLOOR_LENGTH,FLOOR_HEIGHT,FLOOR_WIDTH);
	pVertexData[3].nrm=D3DXVECTOR3(0,1,0);
	pVertexData[3].dif=D3DCOLOR_ARGB(255,255,255,255);
	pVertexData[3].tu=NUM_FLOOR_LENGTH_WRAPS;
	pVertexData[3].tv=NUM_FLOOR_WIDTH_WRAPS;
	//set the next ones for the water
	pVertexData[4].pos=D3DXVECTOR3(-WATER_LENGTH,WATER_HEIGHT,-WATER_WIDTH);
	pVertexData[4].nrm=D3DXVECTOR3(0,1,0);
	pVertexData[4].dif=WATER_COLOR;
	pVertexData[4].tu=0;
	pVertexData[4].tv=0;
	pVertexData[5].pos=D3DXVECTOR3(WATER_LENGTH,WATER_HEIGHT,-WATER_WIDTH);
	pVertexData[5].nrm=D3DXVECTOR3(0,1,0);
	pVertexData[5].dif=WATER_COLOR;
	pVertexData[5].tu=NUM_WATER_LENGTH_WRAPS;
	pVertexData[5].tv=0;
	pVertexData[6].pos=D3DXVECTOR3(-WATER_LENGTH,WATER_HEIGHT,WATER_WIDTH);
	pVertexData[6].nrm=D3DXVECTOR3(0,1,0);
	pVertexData[6].dif=WATER_COLOR;
	pVertexData[6].tu=0;
	pVertexData[6].tv=NUM_WATER_WIDTH_WRAPS;
	pVertexData[7].pos=D3DXVECTOR3(WATER_LENGTH,WATER_HEIGHT,WATER_WIDTH);
	pVertexData[7].nrm=D3DXVECTOR3(0,1,0);
	pVertexData[7].dif=WATER_COLOR;
	pVertexData[7].tu=NUM_WATER_LENGTH_WRAPS;
	pVertexData[7].tv=NUM_WATER_WIDTH_WRAPS;
	

	//we're done modifying the vbuffer, unlock it
	if(FAILED(hr=m_VertexBuffer->Unlock()))
		return(hr);

	//load the textures
	if(FAILED(hr=D3DXCreateTextureFromFile(pDevice,WATER_TEXTURE,&m_WaterTexture)))
		return(hr);
	if(FAILED(hr=D3DXCreateTextureFromFile(pDevice,FLOOR_TEXTURE,&m_FloorTexture)))
		return(hr);

	//return success
	return(S_OK);
}
//renders the waves and floor. uses the vertex buffer
HRESULT CW3DWater::Render(IDirect3DDevice8* pDevice)
{
	//set texture stage states which tell direct3d to wrap the texture
	if(FAILED(hr=pDevice->SetTextureStageState(0,D3DTSS_ADDRESSU,D3DTADDRESS_WRAP)))
		return(hr);
	if(FAILED(hr=pDevice->SetTextureStageState(0,D3DTSS_ADDRESSV,D3DTADDRESS_WRAP)))
		return(hr);

	//render the floor and water
	if(FAILED(hr=pDevice->SetStreamSource(0,m_VertexBuffer,sizeof(WATERVERTEX))))
		return(hr);
	if(FAILED(hr=pDevice->SetVertexShader(D3DFVF_WATERVERTEX)))
		return(hr);
	//set the floor texture and render the floor
	if(FAILED(hr=pDevice->SetTexture(0,m_FloorTexture)))
		return(hr);
	if(FAILED(hr=pDevice->DrawPrimitive(D3DPT_TRIANGLESTRIP,0,2)))
		return(hr);

	//set water render states
	if(FAILED(hr=pDevice->SetRenderState(D3DRS_ALPHABLENDENABLE,true)))
		return(hr);
	pDevice->SetRenderState( D3DRS_ZWRITEENABLE, false );
	pDevice->SetRenderState( D3DRS_SRCBLEND, D3DBLEND_SRCCOLOR );
	pDevice->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_SRCCOLOR );

	//set the texture transform based on the time
	//this animates the water and makes it seem to move with the wind
	D3DXMATRIX matTexture;
	D3DXMatrixIdentity(&matTexture);
	D3DXMatrixRotationYawPitchRoll(&matTexture,0,0,timeGetTime()/10000.0f);
	if(FAILED(hr=pDevice->SetTransform(D3DTS_TEXTURE0,&matTexture)))
		return(hr);

	//tell direct3d that we'll be using texture transformations
	if(FAILED(hr=pDevice->SetTextureStageState(0,D3DTSS_TEXTURETRANSFORMFLAGS,D3DTTFF_COUNT2)))
		return(hr);

	//now set the water texture and render the water
	if(FAILED(hr=pDevice->SetTexture(0,m_WaterTexture)))
		return(hr);
	if(FAILED(hr=pDevice->DrawPrimitive(D3DPT_TRIANGLESTRIP,4,2)))
		return(hr);

	//set renderstates
	pDevice->SetRenderState( D3DRS_ZWRITEENABLE, true );
	pDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, false );

		//tell direct3d that we'll be using texture transformations
	if(FAILED(hr=pDevice->SetTextureStageState(0,D3DTSS_TEXTURETRANSFORMFLAGS,0)))
		return(hr);

	//return success
	return(S_OK);
}
//releses the class's memory
HRESULT CW3DWater::Destroy()
{
	//release the vbuffer
	if(m_VertexBuffer)
	{
		if(FAILED(hr=m_VertexBuffer->Release()))
			return(hr);
		m_VertexBuffer=0;
	}
	//release the textures
	if(m_WaterTexture)
	{
		if(FAILED(hr=m_WaterTexture->Release()))
			return(hr);
		m_WaterTexture=0;
	}
	if(m_FloorTexture)
	{
		if(FAILED(hr=m_FloorTexture->Release()))
			return(hr);
		m_FloorTexture=0;
	}
	//return success
	return(S_OK);
}