//CW3DWorld.cpp
//Brent Gunning
//Copyright 2002 All Rights Reserved

//include header
#include "../headers/CW3DWorld.h"

//constructor/deconstructor
//sets default values and releases memory
CW3DWorld::CW3DWorld()
{
	//set default values for the variables. 
	//most of these are 0's
	hr=0;
}
CW3DWorld::~CW3DWorld()
{
	//release the memory for the class by calling destroy. 
	Destroy();
}
//loads the initilaizes the world. It loads all the files needed and prepares it
//to be rendered with small updates
HRESULT CW3DWorld::Initialize(IDirect3DDevice8* pDevice)
{
	//initialize the trees. it loads and creates the leaves/particles for
	//the trees along the terrain. It loads the trees as points from a file
	//and then uses that data to create the actualt trees
	if(FAILED(hr=m_Trees.Initialize(pDevice)))
		return(hr);

	//initialize the skybox. it loads up the textures for it and sets up a vertex buffer
	//to render the skybox
	if(FAILED(hr=m_SkyBox.Initialize(	SKYBOX_SIZE,SKYBOX_TOP,SKYBOX_BOTTOM,SKYBOX_LEFT,
										SKYBOX_RIGHT,SKYBOX_FRONT,SKYBOX_BACK,pDevice)))
		return(hr);

	//load up the level from file
	if(FAILED(hr=m_Level.Initialize(LEVEL_FILE,LEVEL_REPATH,pDevice)))
		MessageBox(0,"hi","hi",MB_OK);

	//create the water
	if(FAILED(hr=m_Water.Initialize(pDevice)))
		return(hr);

	//initialize the volcano
	sParticleSysProps pp;
	pp.bRemakeParticles=true;
	pp.cEndColor=VOLCANO_END_COLOR;
	pp.cStartColor=VOLCANO_START_COLOR;
	pp.fWeight=VOLCANO_WEIGHT;
	pp.fWeightRand=VOLCANO_WEIGHT_RAND;
	pp.fXSize=VOLCANO_SIZE_X;
	pp.fYSize=VOLCANO_SIZE_Y;
	pp.iLifetime=VOLCANO_LIFETIME;
	pp.iLifetimeRand=VOLCANO_LIFETIME/5;
	pp.iNumParticles=NUM_VOLCANO_PARTICLES;
	pp.vecInitPos=VOLCANO_POSITION;
	pp.vecInitVel=VOLCANO_VELOCITY;
	pp.vecPosRand=VOLCANO_POSITION_RAND;
	pp.vecVelRand=VOLCANO_VELOCITY_RAND;
	pp.vecWind=WIND_DIRECTION;
	if(FAILED(hr=m_Volcano.Initialize(pp,pDevice)))
		return(hr);
	if(FAILED(hr=D3DXCreateTextureFromFileEx(pDevice,"data/particle.bmp",300,300,0,0,D3DFMT_UNKNOWN,
		D3DPOOL_MANAGED,D3DX_DEFAULT,D3DX_DEFAULT,D3DCOLOR_ARGB(255,0,0,0),0,0,&m_VolcTex)))
		return(hr);

	//update the systems's defaults
	for(int i=0;i<500;i++)
	{
		m_Volcano.Process();
	}

	//return success
	return(S_OK);
}
//renders the entire world. This includes terrain, water, and more. This is the main
//function for rendering the world. it takes a camera pointer so it can setup
//billboards and skyboxes and other stuff.
HRESULT CW3DWorld::Render(IDirect3DDevice8* pDevice,CW3DCamera* pCamera)
{
	//next we initialize a view frustum and render the level testing
	//for bounding boxes. we need a matrix of the combined world and projection 
	//matrices to create the frustrum
	D3DXMATRIX matCombined;
	D3DXMatrixMultiply(&matCombined,&pCamera->GetViewMatrix(),&pCamera->GetProjectionMatrix());
	CW3DFrustum frustum;
	frustum.Initialize(matCombined);

	if(FAILED(hr=pDevice->SetTextureStageState(0,D3DTSS_MINFILTER,D3DTEXF_LINEAR)))
		return(hr);
	if(FAILED(hr=pDevice->SetTextureStageState(0,D3DTSS_MAGFILTER,D3DTEXF_LINEAR)))
		return(hr);

	//render the level
	//first we set the world matrix to the identity matrix because
	//we dont' want the level to be transformed like the trees were
	D3DXMATRIX matWorld;
	D3DXMatrixIdentity(&matWorld);
	if(FAILED(hr=pDevice->SetTransform(D3DTS_WORLD,&matWorld)))
		return(hr);
	//now we render the level
	if(FAILED(hr=m_Level.Render(&frustum,LEVEL_TEST_BOUNDING_BOX,LEVEL_TEST_BOUNDING_SPHERE,pDevice)))
		return(hr);

	//render the water
	if(FAILED(hr=m_Water.Render(pDevice)))
		return(hr);

	if(FAILED(hr=pDevice->SetTextureStageState(0,D3DTSS_MINFILTER,D3DTEXF_POINT)))
		return(hr);
	if(FAILED(hr=pDevice->SetTextureStageState(0,D3DTSS_MAGFILTER,D3DTEXF_POINT)))
		return(hr);

	//render the trees. this renders the trees and leaves of them.
	if(FAILED(hr=m_Trees.Render(pCamera->GetRotation(),&frustum,pDevice)))
		return(hr);

		//render the skybox that surrounds the user in a tropical environment
	if(FAILED(hr=m_SkyBox.Render(pCamera->GetPosition(),pDevice)))
		return(hr);
	
	//render the volcano
	pDevice->SetRenderState(D3DRS_ALPHABLENDENABLE,true);
	pDevice->SetRenderState( D3DRS_ZWRITEENABLE,false);
	pDevice->SetTextureStageState( 0, D3DTSS_MINFILTER, D3DTEXF_LINEAR );
	pDevice->SetTextureStageState( 0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR );
	pDevice->SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_ADD );
	pDevice->SetTextureStageState( 0, D3DTSS_ALPHAOP,   D3DTOP_MODULATE);
	pDevice->SetTextureStageState( 1, D3DTSS_COLOROP,   D3DTOP_DISABLE );
	pDevice->SetTextureStageState( 1, D3DTSS_ALPHAOP,   D3DTOP_DISABLE );
	pDevice->SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_MODULATE );
	pDevice->SetRenderState( D3DRS_ZWRITEENABLE, FALSE );
	pDevice->SetRenderState( D3DRS_SRCBLEND, D3DBLEND_ONE );
	pDevice->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_ONE );

	if(FAILED(hr=pDevice->SetTexture(0,m_VolcTex)))
		return(hr);
	if(FAILED(hr=m_Volcano.Process()))
		return(hr);
	if(FAILED(hr=m_Volcano.Render(pDevice,matWorld)))
		return(hr);

	//set previous states back
	pDevice->SetRenderState( D3DRS_ZWRITEENABLE,true);
	pDevice->SetRenderState(D3DRS_ALPHABLENDENABLE,false);

	//return success
	return(S_OK);
}
//releases the world's memory. It calls the sub destroy functions inside of the classes
HRESULT CW3DWorld::Destroy()
{
	//release the trees' memory by calling the destroy function of the class
	if(FAILED(hr=m_Trees.Destroy()))
		return(hr);

	//destroy the skybox
	if(FAILED(hr=m_SkyBox.Destroy()))
		return(hr);

	//now destroy the actual level
	if(FAILED(hr=m_Level.Destroy()))
		return(hr);

	//destroy the wtaer
	if(FAILED(hr=m_Water.Destroy()))
		return(hr);

	//return success
	return(S_OK);
}